### get the DISTRO variable
include $(MACHROOT)/state/dist.mk

### dir where a package will store it's results
RESULTDIR=$(MACHROOT)/result/$(DISTRO)/$(NAME)

work/$(DISTFILE): work cvs
	@(mkdir -p work)
	@if test ! -f work/$(DISTFILE); then \
	  echo "Getting tarball work/$(DISTFILE)"; \
	  cd work && \
	  wget --passive -N $(DISTLOCATION)/$(DISTFILE); \
        else \
          echo "Using work/$(DISTFILE)"; \
	fi

## check if the root can build rpms
rpm-build:
	@if test ! -f $(MACHROOT)/state/rpm-build; then \
	  echo "Prepare the build root for rpm building first !"; \
          echo "Run 'make rpm-build' in the dist dir of your choice."; \
	  exit 1; \
	fi

## build the rpm
rpm: rpm-build pre-goals work/$(DISTFILE) rpm-prerequires rpm-buildrequires rpm-sources result-clear
	@echo Building rpm
	@make lock > /dev/null
	@mkdir -p $(RESULTDIR)
	@echo Starting rpm build
	@if test "x$(SPEC)" = "x"; then \
	  $(CHROOT) $(ROOT) rpmbuild $(RPMFLAGS) -ta /usr/src/rpm/SOURCES/$(DISTFILE) > $(RESULTDIR)/rpm.log 2>&1 || make rpm-fail; \
	else \
	  cp --preserve=t $(SPEC) $(ROOT); \
	  $(CHROOT) $(ROOT) chown root:root $(SPEC); \
	  $(CHROOT) $(ROOT) mv $(SPEC) /usr/src/rpm/SPECS; \
	  $(CHROOT) $(ROOT) rpmbuild $(RPMFLAGS) -ba /usr/src/rpm/SPECS/$(SPEC) > $(RESULTDIR)/rpm.log 2>&1 || make rpm-fail; \
	fi
	@grep "Wrote: " $(RESULTDIR)/rpm.log | cut -d\  -f2 | sort | uniq | \
	  xargs $(CHROOT) $(ROOT) mv --target-directory=/
	mv $(ROOT)/*.rpm $(RESULTDIR)
	@make unlock > /dev/null
	@echo "Build of $(DIST) succeeded."

$(RESULTDIR):
	mkdir -p $(RESULTDIR)

## build rpms and install them all
## rpm-requires: removed from preconds because this would case generated
## -devel packages install their old regular counterpart
## we need to find some way to install necessary reqs though
rpm-install: rpm $(RESULTDIR)
	@echo Installing rpms
	@make lock > /dev/null
	$(RPM) --root $(ROOT) -Uhv `ls $(RESULTDIR)/*.rpm | grep -v \.src\.rpm`
	@make unlock > /dev/null

## when rpm building fails, show why and clean up
rpm-fail:
	@make unlock > /dev/null
	@echo "Build failed."
	@if test -f $(RESULTDIR)/rpm.log; then \
	  echo "Dump of end of rpm building log follows"; \
	  tail -n 20 $(RESULTDIR)/rpm.log; \
	fi
	@echo
	@exit 1

## install all of the rpm's listed as explicit REQUIRES in the Makefile
## FIXME: we list make explicitly as to never have an empty for line which
## always fails
	    ###($(CHROOT) $(ROOT) apt-get -q -y install $$a > /dev/null 2> /dev/null) || echo "Error on apt-get install $$a"; 

rpm-prerequires:
	@make lock > /dev/null
	@echo "Checking for explicitly listed RPMS to install"
	@($(CHROOT) $(ROOT) apt-get -q update > /dev/null 2> /dev/null) || echo "Could not apt-get update"
	@if test "x" != x$(REQUIRES); then \
          echo "Installing pre-requires listed in Makefile : "; \
	  echo "$(REQUIRES)"; \
	  for a in "make $(REQUIRES)"; do \
	    ($(CHROOT) $(ROOT) apt-get -q -y install $$a || echo "Error on apt-get install $$a"); \
	  done; \
        fi
	@make unlock > /dev/null

## install all of the necessary build requires
## filter out any word not starting with a letter from the buildrequires
## this line of perl works for one line but not on a pipe
## perl -ne 'print join (" ", grep {/^[a-zA-Z]\w*$/} split (/ /));'
## the bit of perl down there makes sure we get rid of version bits first,
## ie. stuff like >= alpha9.7 (cdparanoia) which would otherwise bust balls
##rpm-buildrequires: rpm-build work/$(PACKAGE).spec
## FIXME: we don't use this anymore because this statically parses instead
## of having rpm tell us what we're missing (e.g. conditional compiles)
rpm-buildrequires-old: rpm-build rpm-spec
	@echo "Installing BuildRequires listed in spec file"
	@make lock > /dev/null
	@($(CHROOT) $(ROOT) apt-get -q update > /dev/null 2> /dev/null) || echo "Could not apt-get update"
	@grep -i "^BuildRequires:" work/$(PACKAGE).spec | \
          sed 's@BuildRequires:@@' | \
          perl -pe 's/(>=|=|>|<=|<)\W*\w+//' | \
	  sed -e"s/^/ /" -e "s/[[:space:]][^a-zA-Z[:space:]][^[:space:]]*//g" | \
	  xargs $(CHROOT) $(ROOT) apt-get -q -y install
	@make unlock > /dev/null

## try building the spec file (up to prep), and parse the failed requirements
rpm-buildrequires: rpm-build rpm-spec rpm-sources $(RESULTDIR) 
	@echo "Installing necessary packages for building from spec file"
	@make lock > /dev/null
	@($(CHROOT) $(ROOT) apt-get -q update > /dev/null 2> /dev/null) || echo "Could not apt-get update"
	@$(CHROOT) $(ROOT) rpmbuild $(RPMFLAGS) -bc /usr/src/rpm/SPECS/$(PACKAGE).spec > $(RESULTDIR)/rpm.log 2>&1 || \
	grep -i " is needed by " $(RESULTDIR)/rpm.log | \
          sed 's@ is needed by .*@@' | sed 's@^[:space:]*@@' | \
          perl -pe 's/(>=|=|>|<=|<)\W*\w+//' | \
	  sed -e"s/^/ /" -e "s/[[:space:]][^a-zA-Z[:space:]][^[:space:]]*//g" | \
	  xargs $(CHROOT) $(ROOT) apt-get -q -y install
	@make unlock > /dev/null


## install all of the necessary requires
## filter out any word not starting with a letter from the requires
## this line of perl works for one line but not on a pipe
## perl -ne 'print join (" ", grep {/^[a-zA-Z]\w*$/} split (/ /));'
rpm-requires: rpm-build work/$(PACKAGE).spec
	@echo "Installing Requires listed in spec file"
	@make lock > /dev/null
	$(CHROOT) $(ROOT) apt-get -q update
	@grep -i "^Requires:" work/$(PACKAGE).spec | \
          sed 's@Requires:@@' | \
	  sed -e"s/^/ /" -e "s/[[:space:]][^a-zA-Z[:space:]][^[:space:]]*//g" | \
	  xargs $(CHROOT) $(ROOT) apt-get -q -y install
	@make unlock > /dev/null

## copy the dist tarball into the SOURCES dir
## copy all the files in the SOURCES/ dir to /usr/src/rpm/SOURCES
## FIXME: this is good for patches and icons and stuff, but should be
## autocollected from the tarball
rpm-sources:
	@echo "Installing tarball into chroot build area"
	@cp --preserve=t work/$(DISTFILE) $(ROOT)
	@$(CHROOT) $(ROOT) chown root:root $(DISTFILE)
	@$(CHROOT) $(ROOT) mv $(DISTFILE) /usr/src/rpm/SOURCES
	@if test -d SOURCES; then \
	  echo "Copying stuff from SOURCES/ into build root"; \
	  make lock > /dev/null; \
          cp -Lrv --preserver=t SOURCES $(ROOT)/tmp; \
          $(CHROOT) $(ROOT) cp -rv --preserve=t tmp/SOURCES usr/src/rpm; \
	  rm -rf $(ROOT)/tmp/SOURCES; \
	  make unlock > /dev/null; \
	fi

## get the spec out of the dist if $SPEC is not defined in Makefile
## install the spec file in the build root
rpm-spec: work/$(DISTFILE)
	@if test "x$(SPEC)" = "x"; then \
	  echo Extracting spec file from tarball; \
          cd work; \
          tar xzf $(DISTFILE) $(DIST)/$(PACKAGE).spec; \
	  mv $(DIST)/$(PACKAGE).spec .; \
	else \
          cp --preserve=t $(SPEC) work/$(PACKAGE).spec; \
	fi
	@echo "Copying work/$(PACKAGE).spec into build root"; \
	  make lock > /dev/null; \
          cp --preserve work/$(PACKAGE).spec $(ROOT)/tmp; \
	  $(CHROOT) $(ROOT) chown root:root tmp/$(PACKAGE).spec; \
          $(CHROOT) $(ROOT) mv tmp/$(PACKAGE).spec usr/src/rpm/SPECS; \
	  make unlock > /dev/null

## re-generate the spec file from spec.in inside the tarball by reconfiguring
rpm-spec-regen: work/$(DISTFILE) rpm-sources
	@echo Regenerating spec file from tarball\'s configure
	@make lock > /dev/null
##	echo "#\!/bin/bash" > $(ROOT)/regen.sh
	echo "cd /usr/local/src && tar xzf /usr/src/rpm/SOURCES/$(DISTFILE) && \\" > $(ROOT)/regen.sh
	echo "cd $(PACKAGE)-$(VERSION) && ./configure && \\" >> $(ROOT)/regen.sh
	echo "mv $(PACKAGE).spec / && cd .." >> $(ROOT)/regen.sh
##	chmod +x $(ROOT)/regen.sh
	$(CHROOT) $(ROOT) bash regen.sh
	rm $(ROOT)/regen.sh
	@mv $(ROOT)/$(PACKAGE).spec work/$(PACKAGE).spec
	@make unlock > /dev/null

## pre-goals makes sure all of the listed PREGOALS get made and installed first
##
pre-goals:
	@if test "x$(PREGOALS)" != "x"; then \
          echo "Building pre-goals $(PREGOALS)"; \
          for goal in nogoal $(PREGOALS); do \
	    echo $$goal; \
            if test -d ../$$goal; then \
	      pushd ../$$goal; make rpm-install || exit 1; popd; \
	    fi; \
	  done; \
	fi

## check out stuff from cvs if CVSROOT is defined
cvs:
	@if test "x$(CVSROOT)" != "x" && test ! -f work/$(DISTFILE); then \
	  echo "Getting from CVS"; make cvs-dist; fi

## CVSSUBDIR is used in case the actual module to dist is not in the toplevel
## module dir; if it's not defined then it defaults to none so that's good
cvs-dist:
	@echo "Making a tarball from CVS"
	@cd work && cvs -q -d$(CVSROOT) co $(CVSMODULE)  || exit 1; \
        cd $(CVSMODULE)/$(CVSSUBDIR); \
        echo "Running autogen.sh"; ($(AUTOGEN) > /dev/null) || exit 1; \
        echo "Running configure"; ($(CONFIGURE) > /dev/null) || exit 1; \
	echo "Running make dist"; ($(MAKEDIST) > /dev/null) || exit 1
	@find work -name $(DISTFILE) -exec mv -v {} work \;
	@if test ! -f work/$(DISTFILE); then \
          echo "Failed to get work/$(DISTFILE) from CVS !"; \
	  exit 1; \
        fi

work:
	@mkdir -p work

### clear the result dir for this package
result-clear:
	@if test "x$(RESULTDIR)" != "x"; then \
	  mkdir -p $(RESULTDIR); rm -rf $(RESULTDIR)/*; \
        fi

### clear the work dir
work-clear:
	rm -rf work

clear:  work-clear result-clear

help:
	@echo "To make a test build (that doesn't clean), run with RPMFLAGS=-ti"
